﻿// Copyright (c) .NET Foundation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.

namespace Microsoft.Azure.WebJobs.Host.Bindings
{
    /// <summary>
    /// An enumeration of binding order values. These values are used by <see cref="IOrderedValueBinder"/>
    /// when determining which order to bind function out parameters.
    /// <remarks>
    /// Generally parameters are bound in their declared order. However, consider a situation where
    /// a function has output parameters for both a blob and a queue. We want to enqueue the message
    /// AFTER the blob. That way, if the queue message has a blob path in it, there is no race condition
    /// for the message consumer. The <see cref="BindStepOrder"/> declaration for a <see cref="IOrderedValueBinder"/>
    /// allows this ordering to be accomplished.
    /// </remarks>
    /// </summary>
    public enum BindStepOrder
    {
        /// <summary>
        /// Default bind order (i.e. parameter declaration order).
        /// </summary>
        Default = 0,

        /// <summary>
        /// Indicates that a parameter should be bound at a later stage, since it might be enqueueing
        /// a message that references entities generated by other bindings.
        /// </summary>
        Enqueue = 1
    }
}
